class Colors:
    REAL = "mediumpurple"
    SYNTH = "mediumturquoise"
    ALPHA = 0.5


class PlotsSizes:
    LARGE_FIGURE = (16, 10)
    XTRA_LARGE_FIGURE = (20, 10)


class Labels:
    REAL = "Real"
    SYNTH = "Synth"
